(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('svelte/store')) :
	typeof define === 'function' && define.amd ? define(['svelte/store'], factory) :
	(global = global || self, global.asyncable = factory(global.store));
}(this, (function (store) { 'use strict';

	function index (getter, setter = () => {}, stores = []) {

		let resolve;
		const initial = new Promise(res => resolve = res);

		const derived$ = store.derived(stores, values => values);

		const store$ = store.writable(initial, set => {
			return derived$.subscribe(async (values = []) => {
				const value = Promise.resolve(getter(...values));
				set(value);
				resolve(value);
			});
		});

		return {
			subscribe: store$.subscribe,
			async update(reducer) {
				if ( ! setter) return;
				store$.update(async value => {
					value = await value;
					const val = await reducer(shallowCopy(value));
					await setter(val, value);
					return val;
				});
			},
			async set(value) {
				if ( ! setter) return;
				value = await value;
				await setter(value);
				store$.set(Promise.resolve(value));
			},
			get() {
				return store.get(store$);
			}
		};
	}

	function shallowCopy(value) {
		if (typeof value !== 'object' || value === null) return value;
		return Array.isArray(value) ? [ ...value ] : { ...value };
	}

	return index;

})));
